from z3 import *
import sys

SERIAL_LEN = 13
s = Solver()

if len(sys.argv) != 2 or len(sys.argv[1]) >= SERIAL_LEN:
	print("Usage: %s <up to first %d digits of serial>" % (sys.argv[0], SERIAL_LEN-1))
	sys.exit(1)

# Serial variables
serial = [BitVec('c%d' % i, 32) for i in range(SERIAL_LEN)]
for c in serial:
	s.add(c >= 0)
	s.add(c < 10)

for i in range(len(sys.argv[1])):
	c = ord(sys.argv[1][i])-ord('0')
	s.add(serial[i] == c)

# Partial sum
partials = [3]
for i in range(len(serial)-1):
	p = BitVec('p%d' % i, 32)
	s.add(p == partials[-1] + ((2*partials[-1]) ^ (serial[i])))
	partials.append(p)

# Final check
s.add(serial[-1] == (partials[-1] % 10))

# Print model
if s.check() == sat:
	m = s.model()
	res = map(lambda s: m[s].as_long(), serial)
	res = map(lambda n: chr(n+ord('0')), res)
	print(''.join(res))
