#!/usr/bin/env python
import angr

BASE_ADDR = 0x400000
def pie_addr(addr):
    return BASE_ADDR + addr

def hook_nop(state):
    state.regs.rax = 0

# Setup project and patch call
proj = angr.Project('dist/bowrain_581bbadaafd23051a25ccb4adc80b670', load_options={'auto_load_libs': False})
proj.hook(pie_addr(0xFAD), hook_nop, length = 5) # The memset call does nothing of importance and messes up angr (why?)

# Create state and setup symbolic variable
state = proj.factory.blank_state(addr=pie_addr(0x000F46))
ADDR_CHOICE = state.regs.rbp - 0x10
state.mem[ADDR_CHOICE:].dword = state.solver.BVS('choice', 32)

# Find funcptr lookup instruction
sm = proj.factory.simgr(state)
sm.explore(find=pie_addr(0xFBF))

# Pick out the state and add constraint
find_state = sm.found[0].state
find_state.solver.add(find_state.solver.Or(find_state.regs.rdx < 0, find_state.regs.rdx > 7))

# Extract and display results
choice = find_state.regs.rbp - 0x10
print('Choice: %0d' % find_state.solver.eval(find_state.mem[choice:].int64_t.resolved, cast_to=int))
print('RDX: %08x' % find_state.solver.eval(find_state.regs.rdx, cast_to=int))
